// constants
const REMINDER_PROPERTY_DELIMITER = "`";
const REMINDER_SEPARATOR = "~";
const REMINDER_FOX_OCCURRENCE_ONCE = "1";
const REMINDER_TO_BE_COMPLETED_DELIMITER = "@";
const REMINDER_FOX_MONTHLY_WILDCARD_CHAR = '*';
const REMINDER_FOX_MONTHLY_WILDCARD_STR = "*";
const REMINDER_IMPORTANT_FLAG = "!";

const REMINDER_FOX_DOMonth = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];// # of days in Non-Leap year Month 
const REMINDER_FOX_lDOMonth = [31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]; // # of days in Leap year Month 

// configurable preferences   
const REMINDER_FOX_ENABLE_ALERT_PREF = "enableAlertFor"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_NONE = "none"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_TODAY = "today"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_UPCOMING = "upcoming"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_ALL = "all"; 
const REMINDER_FOX_SUSPEND_ALERT_PREF = "SUSPENDED:";

const REMINDER_FOX_ALERT_TIMEOUT_PREF="alertTimeout";
const REMINDER_FOX_ALERT_TIMEOUT_DEFAULT = 120; // number of minutes between alerts

const UPCOMING_REMINDER_DAYS_PREF="upcomingReminderDays";
const UPCOMING_REMINDER_DAYS_DEFAULT = 15;

const REMINDER_FOX_SHOW_STATUS_TEXT="showReminderStatusText";
const REMINDER_FOX_SHOW_STATUS_TEXT_DEFAULT=true;

const REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP="showRemindersInTooltip";
const REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP_DEFAULT=true;

const REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP="showTodosInTooltip";
const REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP_DEFAULT=true;

const REMINDER_FOX_MIGRATED_PREF="migrated";
const REMINDER_FOX_MIGRATED_PREF_VERSION="0.5.5";

const REMINDER_FOX_PREF = "reminderFox";
const REMINDER_FOX_REMINDERS_PREF = "reminders";
const REMINDER_FOX_REMINDERS = REMINDER_FOX_PREF + "." + REMINDER_FOX_REMINDERS_PREF;

const REMINDER_FOX_TODOS_PREF = "todos";
const REMINDER_FOX_TODOS = REMINDER_FOX_PREF + "." + REMINDER_FOX_TODOS_PREF;

const REMINDER_FOX_DEBUG = "debug";
const REMINDER_FOX_LAST_UPDATE = REMINDER_FOX_PREF + "." +REMINDER_FOX_DEBUG + "." + "lastUpdate";
const REMINDER_FOX_LAST_ALERT = REMINDER_FOX_PREF + "." +REMINDER_FOX_DEBUG + "." + "lastAlert";
const REMINDER_FOX_BACKUP_FILE =  REMINDER_FOX_PREF + "." + "backupFile";

const reminderFox_prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

const reminderFox_nsISupportsString = Components.interfaces.nsISupportsString;

/* get unicode string value from preference store */
function getUnicodePref(prefName) {
    return reminderFox_prefs.getComplexValue(prefName, reminderFox_nsISupportsString).data;
}

/* set unicode string value */
function setUnicodePref(prefName,prefValue) {
    var sString = Components.classes["@mozilla.org/supports-string;1"].createInstance(reminderFox_nsISupportsString);
    sString.data = prefValue;
    reminderFox_prefs.setComplexValue(prefName,reminderFox_nsISupportsString,sString);
}


// simple struct to hold both the reminder text and its status
function ReminderContainer( reminderStr, importantStatus ) {
    this.reminder = reminderStr;
    this.important = importantStatus;
}

function reminderFox_isLeapYear(year) { 
	return (((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0)) ? true : false;
}



function reminderFox_getMonthAsInteger(monthTextValue) {
	var prefMonth;
	switch (monthTextValue) {
		case "January":
		prefMonth = 0;
		break;

		case "February":
		prefMonth = 1;
		break;

		case "March":
		prefMonth = 2;
		break;

		case "April":
		prefMonth = 3;
		break;

		case "May":
		prefMonth = 4;
		break;

		case "June":
		prefMonth = 5;
		break;

		case "July":
		prefMonth = 6;
		break;

		case "August":
		prefMonth = 7;
		break;

		case "September":
		prefMonth = 8;
		break;

		case "October":
		prefMonth = 9;
		break;

		case "November":
		prefMonth = 10;
		break;

		case "December":
		prefMonth = 11;
		break;
		
		case "Monthly":
		prefMonth = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
		break;
	}
	return prefMonth;

}


function reminderFox_getMonthAsText(integerValue) {
	var monthAsText;
	
	integerValue = integerValue + "";  // convert to string
	
	switch (integerValue) {
		case "0":
		monthAsText = "January";
		break;

		case "1":
		monthAsText = "February";
		break;

		case "2":
		monthAsText = "March";
		break;

		case "3":
		monthAsText = "April";
		break;

		case "4":
		monthAsText = "May";
		break;

		case "5":
		monthAsText = "June";
		break;

		case "6":
		monthAsText = "July";
		break;

		case "7":
		monthAsText = "August";
		break;

		case "8":
		monthAsText = "September";
		break;

		case "9":
		monthAsText = "October";
		break;

		case "10":
		monthAsText = "November";
		break;

		case "11":
		monthAsText = "December";
		break;
	}
	return monthAsText;
}


function reminderFox_loadDefaultPreferences() {
	var prefsBranch = Components.classes["@mozilla.org/preferences-service;1"]. 
				getService(Components.interfaces.nsIPrefService). 
				getBranch(REMINDER_FOX_PREF + ".");

	// If the reminders preferences do not exist create them with default values
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_REMINDERS_PREF, "");
	}	
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ENABLE_ALERT_PREF);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_ENABLE_ALERT_PREF, REMINDER_FOX_ENABLE_ALERT_PREF_ALL);
	}
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ALERT_TIMEOUT_PREF);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_ALERT_TIMEOUT_PREF, REMINDER_FOX_ALERT_TIMEOUT_DEFAULT);
	}
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + UPCOMING_REMINDER_DAYS_PREF);
	} catch(e) {
		prefsBranch.setCharPref(UPCOMING_REMINDER_DAYS_PREF, UPCOMING_REMINDER_DAYS_DEFAULT);
	}
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_TODOS_PREF, "");
	}	
	try {
		reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_STATUS_TEXT);
	} catch(e) {
		prefsBranch.setBoolPref(REMINDER_FOX_SHOW_STATUS_TEXT, REMINDER_FOX_SHOW_STATUS_TEXT_DEFAULT);
	}	
	
	try {
		reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP);
	} catch(e) {
		prefsBranch.setBoolPref(REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP, REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP_DEFAULT);
	}	

	try {
		reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP);
	} catch(e) {
		prefsBranch.setBoolPref(REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP, REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP_DEFAULT);
	}			
		
	try {
		var oldVersionNumber = reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_MIGRATED_PREF);
		if ( oldVersionNumber != REMINDER_FOX_MIGRATED_PREF_VERSION ) {
			// this is an older version; we need to handle migration code	
			reminderFox_handleMigration( oldVersionNumber );				
		}		
	} catch(e) {
		// this is an older version; we need to handle migration code		
		reminderFox_handleMigration("0.0");
	}
}

function reminderFox_sortNewReminder( reminders, newreminder, monthInt, dayInt  ) {
	return reminderFox_sortNewReminder( reminders, newreminder, monthInt, dayInt, false );
}

function reminderFox_sortNewReminder( reminders, newreminder, monthInt, dayInt, addUniqueOnly  ) {
	// We go through each reminder and then compare that reminder's month with 
	// the reminder-to-be-added month, and if the new month is less, add there;
	// if the months are equal, then compare the days and add appropriately. 
	if (reminders != "") {
	 	var index = 0;
		var endIndex = reminders.indexOf( REMINDER_SEPARATOR );
		if ( endIndex == -1 ) {
			endIndex = reminders.length ;
		}
		var added = false;
		while ( endIndex != -1 && !added ) {
			var currentReminder;
			if ( endIndex == reminders.length - 1 ) {
				currentReminder = reminders.substring( index, endIndex + 1 );
			}
			else {
				currentReminder = reminders.substring( index, endIndex );
			}
			var reminderArray = currentReminder.split(REMINDER_PROPERTY_DELIMITER);

			var myMonth;
			var reminderMon;
			// treat Monthly reminders as coming at the end of the list
			if ( monthInt == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
				myMonth = 12;
			}
			else {
			    myMonth = parseInt( monthInt );
			}
			if ( reminderArray[0] == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) { 
				reminderMon = 12;
			}
			else {
				reminderMon = parseInt(reminderArray[0]);
			}
		
			// add current reminder before reminders from later months
			if (  myMonth < reminderMon) { 
				var leftSideOfReminders = reminders.substring(0, index );						
				var rightSideOfReminders = reminders.substring(index ,  reminders.length);					
				reminders = leftSideOfReminders + newreminder + REMINDER_SEPARATOR + rightSideOfReminders;
				added = true;
			}
			// if they have the same month, then compare dates
			else if ( reminderMon == myMonth ) {
				var myDay = parseInt(dayInt);
				var reminderDay = parseInt(reminderArray[1]);
				if ( myDay < reminderDay ) {
					var leftSideOfReminders = reminders.substring(0, index );							
					var rightSideOfReminders = reminders.substring(index ,  reminders.length);					
					reminders = leftSideOfReminders + newreminder + REMINDER_SEPARATOR + rightSideOfReminders;
					added = true;
				}
				else if ( myDay == reminderDay ) {
					var newReminderArray = newreminder.split(REMINDER_PROPERTY_DELIMITER);
					if ( addUniqueOnly && newReminderArray[2] == reminderArray[2] ) {
						// this reminder is already in the list; just ignore it
						added = true;
					}
					else {
						// if this is an Important reminder, add it before other reminders of the same day
						var indexImportant = newreminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
						if ( indexImportant != -1 ) {
							var leftSideOfReminders = reminders.substring(0, index );							
							var rightSideOfReminders = reminders.substring(index ,  reminders.length);					
							reminders = leftSideOfReminders + newreminder + REMINDER_SEPARATOR + rightSideOfReminders;
							added = true;
						}
					}
				}
			}
			index = endIndex + 1;
			endIndex = reminders.indexOf( REMINDER_SEPARATOR, index );		
			if ( index != reminders.length && endIndex == -1 ) {
				endIndex = reminders.length - 1;
			}
		}
		
		// if the reminder hasn't been added yet, then place it at the end
		// of the list
		if ( !added ) {
			reminders += REMINDER_SEPARATOR + newreminder;
		}	
	}	
	else {
		reminders += newreminder;
	}	
	return reminders;
}

function reminderFox_sortNewTodo( todos, newtodo ) {
	if (todos != "") {
	 	var index = 0;
		var endIndex = todos.indexOf( REMINDER_SEPARATOR );
		if ( endIndex == -1 ) {
			endIndex = todos.length - 1;
		}
		var added = false;
		
		var newTodoArray = newtodo.split(REMINDER_PROPERTY_DELIMITER);
		
		var isImportant = false;
		var showInTodo = false;
		// check for any options that are specified, and mark them in the columns 
		if ( newTodoArray.length >= 2 ) {
            for ( var j = 1; j < newTodoArray.length; j++ ) {
                var val = newTodoArray[j];
                if ( val == REMINDER_IMPORTANT_FLAG ) {
                    isImportant = true;        
                }
                else if ( val == REMINDER_TO_BE_COMPLETED_DELIMITER ) {
                    showInTodo = true;        
                } 
            }
        }
     
		// if current todo is not important, add to end of list...
        if ( isImportant ) {
            todos = newtodo + REMINDER_SEPARATOR + todos;
        	added = true;
    
        }
        else {
        	// run through until find one that's not important
    		//todos = todos + REMINDER_SEPARATOR + newtodo;
        	//added = true;
        	
        var index = 0;
		var endIndex = todos.indexOf( REMINDER_SEPARATOR );
		if ( endIndex == -1 ) {
			endIndex = todos.length - 1;
		}
		//var added = false;
		while ( endIndex != -1 && !added ) {
			var currentReminder = todos.substring( index, endIndex );
			reminderArray = currentReminder.split(REMINDER_PROPERTY_DELIMITER);
			
			var curImportant = false;
			var curShowInTodo = false;
			if ( reminderArray.length >= 2 ) {
	            for ( var j = 1; j < reminderArray.length; j++ ) {
	                var val = reminderArray[j];
	                if ( val == REMINDER_IMPORTANT_FLAG ) {
	                    curImportant = true;        
	                }
	                else if ( val == REMINDER_TO_BE_COMPLETED_DELIMITER ) {
	                    curShowInTodo = true;        
	                } 
	            }
	        }
	        
	        if ( !curImportant ) {
	        	if ( showInTodo ) {
	        		// add here
	        		var leftSideOfReminders = todos.substring(0, index );						
					var rightSideOfReminders = todos.substring(index ,  todos.length);					
					todos = leftSideOfReminders + newtodo + REMINDER_SEPARATOR + rightSideOfReminders;
	        		
	        		added = true;
	        	}
	        	else {
	        		if ( !curShowInTodo ) {
	        			// add here 
	        			var leftSideOfReminders = todos.substring(0, index );						
						var rightSideOfReminders = todos.substring(index ,  todos.length);					
						todos = leftSideOfReminders + newtodo + REMINDER_SEPARATOR + rightSideOfReminders;
	        			
	        			added = true;
	        		}
	        	}
	        
	        }


			index = endIndex + 1;
			endIndex = todos.indexOf( REMINDER_SEPARATOR, index );		
			if ( index != todos.length && endIndex == -1 ) {
				endIndex = todos.length -1;
			}
		}

      }
		
		// if the reminder hasn't been added yet, then place it at the end
		// of the list
		if ( !added ) {
			todos += REMINDER_SEPARATOR + newtodo;
		}			
		
	}	
	else {
		todos += newtodo;
	}	
	return todos;
}


function reminderFox_sortNewTodoBefore( todos, newtodo, todoBefore ) {
	if (todos != "" && todoBefore != null ) {
	 	var index = 0;
		var endIndex = todos.indexOf( REMINDER_SEPARATOR );
		if ( endIndex == -1 ) {
			endIndex = todos.length - 1;
		}
		var added = false;
		
		var newTodoArray = newtodo.split(REMINDER_PROPERTY_DELIMITER);
		
        var index = 0;
		var endIndex = todos.indexOf( REMINDER_SEPARATOR );
		if ( endIndex == -1 ) {
			endIndex = todos.length - 1;
		}

		while ( endIndex != -1 && !added ) {
			var currentReminder = todos.substring( index, endIndex );
			reminderArray = currentReminder.split(REMINDER_PROPERTY_DELIMITER);
			if ( reminderArray[0] == todoBefore ) {
                var leftSideOfReminders = todos.substring(0, index );						
                var rightSideOfReminders = todos.substring(index ,  todos.length);					
                todos = leftSideOfReminders + newtodo + REMINDER_SEPARATOR + rightSideOfReminders;         
                added = true;
             }

			index = endIndex + 1;
			endIndex = todos.indexOf( REMINDER_SEPARATOR, index );		
			if ( index != todos.length && endIndex == -1 ) {
				endIndex = todos.length;
			}
		}

				
		// if the reminder hasn't been added yet, then place it at the end
		// of the list
		if ( !added ) {
			todos += REMINDER_SEPARATOR + newtodo;
		}			
		
	}	
	else {
			todos += REMINDER_SEPARATOR + newtodo;
	}	
	return todos;
}
 
 function reminderFox_getValidDateForMonth( year, month, date ) {
	var lastValidDate = date;	
	var lastDayOfMonth = oeDatePicker.getLastDayOfMonth( year, month );
	if ( date > lastDayOfMonth ) {
		lastValidDate = lastDayOfMonth;
	}
	return lastValidDate;
}
 
function reminderFox_handleMigration(oldVersion) {
		// this is an older version; we need to handle migration code		
		var prefsBranch = Components.classes["@mozilla.org/preferences-service;1"]. 
				getService(Components.interfaces.nsIPrefService). 
				getBranch(REMINDER_FOX_PREF + ".");
		
		var reminders = "";
		try {
			reminders = reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
		}
		catch (e) {
		}
		
		if ( reminders != "" ) {
			if ( oldVersion < "0.3" ) {
				// replace old delimeters
				reminders = reminders.replace( /\$/g, REMINDER_PROPERTY_DELIMITER );
				reminders = reminders.replace( /,/g, REMINDER_SEPARATOR );
			}
			
			// now sort all of the reminders
			var newReminderStr = "";			
			if ( reminders.lastIndexOf( REMINDER_SEPARATOR ) != reminders.length - 1 ) {
				reminders = reminders + REMINDER_SEPARATOR;
			}
			
			var remindersArray = reminders.split(REMINDER_SEPARATOR);

			for(var i = 0; i < remindersArray.length; i++) {
				var newreminder = remindersArray[i];
				var reminderArray = newreminder.split(REMINDER_PROPERTY_DELIMITER);			
				newReminderStr = reminderFox_sortNewReminder( newReminderStr, newreminder, reminderArray[0], reminderArray[1]  );				
			}
			
			reminders = newReminderStr;
			if ( reminders.lastIndexOf( REMINDER_SEPARATOR ) == reminders.length - 1 ) {
				reminders = reminders.substring( 0, reminders.length - 1);
			}
			reminderFox_prefs.setCharPref(REMINDER_FOX_REMINDERS, reminders);
			
		}
		prefsBranch.setCharPref(REMINDER_FOX_MIGRATED_PREF, REMINDER_FOX_MIGRATED_PREF_VERSION);
		
}